from presidio_analyzer import Pattern, PatternRecognizer
from typing import Optional, List, Tuple


class ItVatCodeRecognizer(PatternRecognizer):
    """
    Recognizes Italian VAT code using regex and checksum.

    For more information about italian VAT code:
        https://en.wikipedia.org/wiki/VAT_identification_number#:~:text=%5B2%5D)-,Italy,-Partita%20IVA

    :param patterns: List of patterns to be used by this recognizer
    :param context: List of context words to increase confidence in detection
    :param supported_language: Language this recognizer supports
    :param supported_entity: The entity this recognizer can detect
    :param replacement_pairs: List of tuples with potential replacement values
    for different strings to be used during pattern matching.
    This can allow a greater variety in input, for example by removing dashes or spaces.
    """

    # Class variables
    PATTERNS = [
        Pattern(
            "IT Vat code (piva)",
            r"\b([0-9][ _]?){11}\b",
            0.1,
        )
    ]

    CONTEXT = ["piva", "partita iva", "pi"]

    def __init__(
        self,
        patterns: Optional[List[Pattern]] = None,
        context: Optional[List[str]] = None,
        supported_language: str = "it",
        supported_entity: str = "IT_VAT_CODE",
        replacement_pairs: Optional[List[Tuple[str, str]]] = None,
    ):
        self.replacement_pairs = (
            replacement_pairs
            if replacement_pairs
            else [("-", ""), (" ", ""), ("_", "")]
        )
        patterns = patterns if patterns else self.PATTERNS
        context = context if context else self.CONTEXT
        super().__init__(
            supported_entity=supported_entity,
            patterns=patterns,
            context=context,
            supported_language=supported_language,
        )

    def validate_result(self, pattern_text: str) -> bool:
        """
        Validate the pattern logic e.g., by running checksum on a detected pattern.

        :param pattern_text: the text to validated.
        Only the part in text that was detected by the regex engine
        :return: A bool indicating whether the validation was successful.
        """

        # Pre-processing before validation checks
        text = self.__sanitize_value(pattern_text, self.replacement_pairs)

        # Edge-case that passes the checksum even though it is not a
        # valid italian vat code.
        if text == "00000000000":
            return False

        x = 0
        y = 0

        for i in range(0, 5):
            x += int(text[2 * i])
            tmp_y = int(text[2 * i + 1]) * 2
            if tmp_y > 9:
                tmp_y = tmp_y - 9
            y += tmp_y

        t = (x + y) % 10
        c = (10 - t) % 10

        if c == int(text[10]):
            result = True
        else:
            result = False

        return result

    @staticmethod
    def __sanitize_value(text: str, replacement_pairs: List[Tuple[str, str]]) -> str:
        for search_string, replacement_string in replacement_pairs:
            text = text.replace(search_string, replacement_string)
        return text



from presidio_analyzer import (
    AnalyzerEngine, 
    RecognizerRegistry, 
    PatternRecognizer
    )
from presidio_analyzer.nlp_engine import NlpEngineProvider
from presidio_analyzer.predefined_recognizers import (
    PhoneRecognizer, 
    SpacyRecognizer, 
    #ItFiscalCodeRecognizer, 
    #ItVatCodeRecognizer
    )

from presidio_anonymizer import AnonymizerEngine
from presidio_anonymizer.entities import OperatorConfig

from anon_testo import it_fiscal_code_recognizer, it_vat_code

import json

def nlpInput(input:dict):
#REQUIRES: Valid GateNLP document as input
#MODIFIES: input
#EFFECTS: Deletes PII from the annotations and returns them in a DenyList ready to be used by Presidio
    parsedText = input
    denyList = []
    try:
        text = parsedText['annotation_sets']["entities"]["annotations"]
        for annotazione in text:
            if annotazione["features"]["ner"]["type"] != "articolo_di_legge" and annotazione["features"]["ner"]["type"] != "data":
                for parola in annotazione["features"]["ner"]["normalized_text"].split():
                    if parola not in denyList:
                        denyList.append(parola)
                annotazione["features"]["title"] = ""
                annotazione["features"]["ner"]["normalized_text"]= "" 
        parsedText['annotation_sets']["entities"]["annotations"] = text
                
        for parola in parsedText['features']["parte"].lower().split():
            denyList.append(parola)

        for parola in parsedText['features']["controparte"].lower().split():
            denyList.append(parola)

        for parolaGiudice in parsedText['features']["nomegiudice"].lower().split():
            denyList.append(parolaGiudice)   

        codFiscaleGiudice = parsedText['features']["cf_giudice"]
        denyList.append(codFiscaleGiudice)

        codFiscaleSender = parsedText['features']["senderid"]    
        denyList.append(codFiscaleSender)

    except:
        print("L'input non è in un formato supportato")
    
    return [denyList, parsedText]

def analisiTesto(denyList:list,testo:json):
#REQUIRES: Validated GateNLP document and DenyList as inputs
#MODIFIES: //
#EFFECTS: Creates the Analyzer and identifies PII via DenyList and other Recognizers
    configuration = {
        "nlp_engine_name": "spacy",
        "models": [
        {"lang_code": "it", "model_name": "it_core_news_lg"},
        ],
    }
    registry = RecognizerRegistry()

    if len(denyList) > 0:
        annotationList_recognizer = PatternRecognizer(supported_language="it", supported_entity= "PERSON", deny_list= denyList)
        registry.add_recognizer(annotationList_recognizer)
    
    telephone_recognizer_italian = PhoneRecognizer(supported_language= "it")
    spacyDefault_recognizer_italian = SpacyRecognizer(supported_language= "it", context= ["avv. ", "sig. "])
    fiscalCode_italian = it_fiscal_code_recognizer.ItFiscalCodeRecognizer(context="C.F. ")
    pIva_italian = it_vat_code.ItVatCodeRecognizer(context="C.F. ")
    
    registry.add_recognizer(telephone_recognizer_italian)
    registry.add_recognizer(spacyDefault_recognizer_italian)
    registry.add_recognizer(fiscalCode_italian)
    registry.add_recognizer(pIva_italian)

    provider = NlpEngineProvider(nlp_configuration = configuration)
    nlp_engine_with_italian = provider.create_engine()

    # Passiamo il provider all'engine
    analyzer = AnalyzerEngine(
    nlp_engine = nlp_engine_with_italian,
    registry = registry,
    supported_languages = "it")

    entity_list = ["PERSON", "IT_FISCAL_CODE", "IT_VAT_CODE", "ORGANIZATION", "LOCATION"]
    
    try:
        results = analyzer.analyze(text = testo["text"], 
                           language = "it", 
                           entities = entity_list,
                           score_threshold = 0.35,
                           return_decision_process= True)
        return (results)
    except:
        print("Analisi fallita")

def anonimTesto(testo:json, risultatiAnalisi:list):
#REQUIRES: Validated GateNLP document text and analysis result as input
#MODIFIES: Validated GateNLP document
#EFFECTS: Creates the Anonymizer and masks the identified PIIs
    engine = AnonymizerEngine()
    # Rinomino gli operatori da quelli di Default ad una versione in italiano
    it_operators = {
        "PERSON": OperatorConfig("mask", {"masking_char": "-", "chars_to_mask": 30, "from_end": False}),
        "LOCATION": OperatorConfig("mask", {"masking_char": "-", "chars_to_mask": 30, "from_end": True}),
        "ORGANIZATION": OperatorConfig("mask", {"masking_char": "-", "chars_to_mask": 30, "from_end": False}),
        "PHONE_NUMBER": OperatorConfig("mask", {"masking_char": "-", "chars_to_mask": 14, "from_end": True}),
        "IT_FISCAL_CODE": OperatorConfig("mask", {"masking_char": "-", "chars_to_mask": 16, "from_end": True}),
        "IT_VAT_CODE": OperatorConfig("mask", {"masking_char": "-", "chars_to_mask": 11, "from_end": True}),
        }

    result = engine.anonymize(testo["text"], risultatiAnalisi, operators = it_operators)

    return result.text

def nlpOutput(nlpRipulito:json, anonimTesto:str):
#REQUIRES: Validated GateNLP document text and anonymized result as input
#MODIFIES: inputNLP
#EFFECTS: Returns the new anonymized GateNLP
    try:
        documentoAnonimizzato = nlpRipulito
        documentoAnonimizzato["text"] = anonimTesto

        return documentoAnonimizzato
    except:
        print("Ritorno del gateNLP modificato fallito")




from setuptools import setup

setup(
    name='anon_testo',
    version='0.1.6',    
    description='Anonimizzatore di documenti GateNLP',
    url='https://github.com/RafVale/anon_testo',
    author='Raffaele Valendino',
    author_email='raffaele.valendino@gmail.com',
    license='MIT',
    packages=['anon_testo'],
    install_requires=['spacy <= 3.2.0',
                    'presidio-analyzer <= 2.2.25',
                    'presidio-anonymizer <= 2.2.25',
                    'typing-inspect==0.8.0',
                    'typing_extensions==4.5.0'
                      ],

    classifiers=[
        'Development Status :: 1 - Planning',
        'Intended Audience :: Education',
        'License :: OSI Approved :: MIT License',  
        'Operating System :: Microsoft :: Windows :: Windows 10',        
        'Programming Language :: Python :: 3.9',
    ],
)
